@@ -1,5 +1,5 @@
 package QBit::QueryData::Function::DISTINCT;
-$QBit::QueryData::Function::DISTINCT::VERSION = '0.010';
+$QBit::QueryData::Function::DISTINCT::VERSION = '0.011';
 use qbit;
 
 use base qw(QBit::QueryData::Function);
@@ -14,13 +14,20 @@ sub init {
 
     $self->SUPER::init();
 
+    return FALSE if $self->has_errors();
+
+    $self->{'PATH'}           = $self->qd->_get_path($self->args->[0]);
     $self->{'__MAIN_FIELD__'} = $self->args->[0];
 }
 
 sub process {
     my ($self, $row) = @_;
 
-    return $self->qd->get_field_value_by_path($row, $row, undef, @{$self->qd->_get_path($self->{'__MAIN_FIELD__'})});
+    return
+        '        $new_row->{'
+      . $self->qd->quote($self->field) . '} = '
+      . $self->qd->_get_field_code_by_path('$row', $self->{'PATH'}) . ';
+';
 }
 
 sub DESTROY {$COUNT_THIS_OBJECT--}
